pg.ddx.io  pgsql-committers@postgresql.org mailing list archive  
help / color / mirror / Atom feed
From: Andres Freund <andres@anarazel.de>
To: pgsql-committers@lists.postgresql.org
Subject: pgsql: Reduce branches in heapgetpage()'s per-tuple loop
Date: Sun, 07 Apr 2024 07:29:18 +0000
Message-ID: <E1rtMxq-000wGZ-6c@gemulon.postgresql.org> (raw)

Reduce branches in heapgetpage()'s per-tuple loop

Until now, heapgetpage()'s loop over all tuples performed some conditional
checks for each tuple, even though condition did not change across the loop.

This commit fixes that by moving the loop into an inline function. By calling
it with different constant arguments, the compiler can generate an optimized
loop for the different conditions, at the price of two per-page checks.

For cases of all-visible tables and an isolation level other than
serializable, speedups of up to 25% have been measured.

Reviewed-by: John Naylor <johncnaylorls@gmail.com>
Reviewed-by: Zhang Mingli <zmlpostgres@gmail.com>
Tested-by: Quan Zongliang <quanzongliang@yeah.net>
Discussion: https://postgr.es/m/20230716015656.xjvemfbp5fysjiea@awork3.anarazel.de
Discussion: https://postgr.es/m/2ef7ff1b-3d18-2283-61b1-bbd25fc6c7ce@yeah.net

Branch
------
master

Details
-------
https://git.postgresql.org/pg/commitdiff/a97bbe1f1df9eba0b18207c321c67de80b33db16

Modified Files
--------------
src/backend/access/heap/heapam.c | 102 ++++++++++++++++++++++++++++-----------
1 file changed, 74 insertions(+), 28 deletions(-)



view thread (3+ messages)  latest in thread

Message-ID: <E1rtMxq-000wGZ-6c@gemulon.postgresql.org>
Permalink:  ../E1rtMxq-000wGZ-6c@gemulon.postgresql.org/
Also on:    postgresql.org/message-id/E1rtMxq-000wGZ-6c@gemulon.postgresql.org

 · 

reply

Reply instructions:

You may reply publicly to this message via plain-text email
using any one of the following methods:

* Reply to all the recipients using the --to and --cc options:
  reply via email

  To: pgsql-committers@postgresql.org
  Cc: andres@anarazel.de, pgsql-committers@lists.postgresql.org
  Subject: Re: pgsql: Reduce branches in heapgetpage()'s per-tuple loop
  In-Reply-To: <E1rtMxq-000wGZ-6c@gemulon.postgresql.org>

* Save the following mbox file, import it into your mail client,
  and reply-to-all from there: mbox

This inbox is served by DDX for PostgreSQL; see mirroring instructions
for how to clone and mirror all data and code used for this inbox